'''
Script that prints information for every question answered by a profile
from quickmatch that contains the text "marijuana"(probably not a whole
lot, but you get the idea).
'''
import okcupyd

u = okcupyd.User()
p = u.quickmatch()

print('Questions for {0}'.format(p.username))
print('----------')
for q in p.questions:
    if 'marijuana' in q.text.lower():
        print(q.text)
        print(q.user_answer)
        if len(q.explanation):
            print(q.explanation)
        print('')



import mock
import pytest

from okcupyd.search import SearchManager, SearchParameterBuilder, search
from okcupyd.profile import Profile
from okcupyd.session import Session
from . import util


class TestSearch(object):

    @pytest.fixture
    def search_manager(self, session):
        return SearchManager(Session(), looking_for='everybody')

    @util.use_cassette('search_age_filter')
    def test_age_filter(self, search_manager):
        age = 22
        search_manager.set_options(age_min=age, age_max=age)
        profile, = search_manager.get_profiles(count=1)
        assert profile.age == age

    @util.use_cassette('search')
    def test_match_card_extractor(self, search_manager, request):
        search_manager.set_options(keywords='hannahmv', looking_for='everybody')
        profiles = search_manager.get_profiles(count=1)
        assert len(profiles) == 1

        # TODO(IvanMalison): This is crap -- clean this up.
        first_profile = profiles[0]
        assert first_profile.username.lower() == 'hannahmv'.lower()
        assert first_profile.age == 27
        assert first_profile.location == 'Astoria, NY'
        assert isinstance(first_profile.match_percentage, int)
        assert isinstance(first_profile.enemy_percentage, int)
        assert first_profile.id == '10558689643648617771'
        assert first_profile.contacted == False
        assert first_profile.rating == 0

    @util.use_cassette('search_count')
    def test_count_variable(self, search_manager, request):
        search_manager.set_options(looking_for='everybody')
        profiles = search_manager.get_profiles(count=14)
        assert len(profiles) == 14

        for profile in profiles:
            profile.username
            profile.age
            profile.location
            profile.match_percentage
            profile.enemy_percentage
            profile.id
            profile.rating
            profile.contacted

    @util.use_cassette('search_location_filter')
    def test_location_filter(self, search_manager):
        location = 'Portland, OR'
        search_manager.set_options(location=location, radius=25)
        profile, = search_manager.get_profiles(count=1)
        assert profile.location == 'Portland, OR'

    @util.use_cassette('search_function')
    def test_search_function(self, session):
        profile, = search(session, count=1)
        assert isinstance(profile, Profile)

        profile.username
        profile.age
        profile.location
        profile.match_percentage
        profile.enemy_percentage
        profile.id
        profile.rating
        profile.contacted


@mock.patch('okcupyd.helpers.get_locid', return_value=2)
def test_construction_of_all_search_parameters(mock_get_locid):
    spb = SearchParameterBuilder()
    spb.set_options(location='new york, ny', religion='buddhist',
                              height_min=66, height_max=68, looking_for='everybody',
                              smokes=['no', 'trying to quit'], age_min=18, age_max=24,
                              radius=12, order_by='MATCH', last_online=1234125,
                              status='single', drugs=['very_often', 'sometimes'],
                              job=['retired'], education=['high school'],
                              income='less than $20,000', monogomy='monogamous',
                              diet='vegan', ethnicity=['asian', 'middle eastern'])

    print(spb.build(mock.Mock(), count=10))



'''
Sample script that gives a 5-star rating to the profiles of all
24-year-old straight/gay/bi women/men (depending on your
gender/orientation) in Minneapolis who like the show Arrested
Development.
'''

import okcupyd

u = okcupyd.User()
profiles = u.search(location='minneapolis, mn', keywords='arrested development',
                    age_min=24, age_max=24, number=1000)
for profile in profiles:
    u.rate(profile, 5)



import mock
import contextlib2 as contextlib
import pytest
from vcr.cassette import Cassette, CassetteContextDecorator

from . import util
from okcupyd import settings
from okcupyd import util as okcupyd_util
from okcupyd.session import Session


def pytest_addoption(parser):
    okcupyd_util.add_command_line_options(parser.addoption, use_short_options=False)
    parser.addoption('--live', dest='skip_vcrpy', action='store_true', default=False,
                     help="Skip the patching of http requests in tests. "
                     "USE WITH CAUTION. This will interact with the okcupid "
                     "website and send messages with any provided user credentials.")
    parser.addoption('--scrub', dest='scrub', action='store_true', default=False,
                     help="USE WITH CAUTION. Don't scrub PII from "
                     "http requests/responses. This is useful for recording cassetes.")
    parser.addoption('--resave', dest='resave_cassettes',
                     action='store_true', default=False,
                     help="Resave cassettes. Use to retoractively scrub cassettes.")
    parser.addoption('--cassette-mode', dest='cassette_mode', action='store',
                     default='once', help="Accept new requests in all tests.")


def patch(option, *patches, **kwargs):
    negate = kwargs.get('negate', False)
    @pytest.yield_fixture(autouse=True, scope='session')
    def patch_conditionally(request):
        condition = bool(request.config.getoption(option))
        if negate: condition = not condition
        if condition:
            with contextlib.ExitStack() as exit_stack:
                for patch in patches:
                    exit_stack.enter_context(patch)
                yield
        else:
            yield
    return patch_conditionally


patch_settings = patch('credentials_modules',
                       mock.patch.object(settings, 'USERNAME', 'username'),
                       mock.patch.object(settings, 'PASSWORD', 'password'),
                       mock.patch.object(settings, 'AF_USERNAME', 'username'),
                       mock.patch.object(settings, 'AF_PASSWORD', 'password'),
                       mock.patch.object(settings, 'DELAY', 0), negate=True)
patch_save = patch('resave_cassettes',
                   mock.patch.object(
                       Cassette, '_save',
                       okcupyd_util.n_partialable(Cassette._save)(force=True)))
patch_use_cassette_enabled = patch('skip_vcrpy',
                                   mock.patch.object(CassetteContextDecorator,
                                                     '__enter__'),
                                   mock.patch.object(CassetteContextDecorator,
                                                     '__exit__'))
patch_vcrpy_filters = patch('scrub',
                            mock.patch.object(util, 'SHOULD_SCRUB', False),
                            negate=True)


@pytest.fixture(autouse=True, scope='session')
def set_vcr_options(request):
    util.okcupyd_vcr.record_mode = request.config.getoption('cassette_mode')


@pytest.yield_fixture(autouse=True, scope='session')
def process_command_line_args(request):
    okcupyd_util.handle_command_line_options(request.config.option)
    if request.config.getoption('skip_vcrpy') or request.config.getoption('credentials_modules'):
        with mock.patch.object(util, 'TESTING_USERNAME', settings.USERNAME):
            yield
    else:
        yield


@pytest.fixture
def session():
    with util.use_cassette('session_success'):
        return Session.login()



'''
Sample script that prints the first message thread in your inbox, and
sends a response message to the sender.
'''

import okcupyd

u = okcupyd.User()
thread = u.inbox[0]
print(thread.messages)
u.message(thread.correspondent, 'Thank you for that highly informative message!')
thread.reply('This is another way to send a message')



from lxml import etree

from okcupyd import xpath


def test_selected_attribute():
    node = xpath.XPathNode(element='element', selected_attribute='value')
    assert node.xpath == '//element/@value'

    tree = etree.XML("<top><container><element value='1'>"
                 "</element><element value='2'></element></container></top>")
    builder = xpath.xpb.container.element.select_attribute_('value')

    assert builder.xpath == './/container//element/@value'
    assert builder.apply_(tree) == ['1', '2']

    assert xpath.xpb.element.select_attribute_('value', elem=tree) == ['1', '2']




import itertools

from lxml import html

from . import helpers
from . import util
from .xpath import xpb


class Question(object):

    def __init__(self, id_, text, user_answer, explanation):
        self.id = id_
        self.text = text
        self.user_answer = user_answer
        self.explanation = explanation

    def __repr__(self):
        return '<Question: {0}>'.format(self.text)


class QuestionFetcher(object):

    step = 10

    @classmethod
    def build(cls, session, username):
        return cls(QuestionHTMLFetcher(session, username), QuestionProcessor(session))

    _page_data_xpb = xpb.div.with_class('pages_data')
    _current_page_xpb = _page_data_xpb.input(id='questions_pages_page').select_attribute_('value')
    _total_page_xpb = _page_data_xpb.input(id='questions_pages_total').select_attribute_('value')

    def __init__(self, html_fetcher, processor):
        self._html_fetcher = html_fetcher
        self._processor = processor

    def _pages_left(self, tree):
        return int(self._current_page_xpb.one_(tree)) < int(self._total_page_xpb.one_(tree))

    def fetch(self, start_at=0, get_at_least=None, id_to_existing=None):
        current_offset = start_at + 1
        generators = []
        while True:
            tree = html.fromstring(self._html_fetcher.fetch(current_offset))
            generators.append(self._processor.process(tree))
            if not self._pages_left(tree):
                break
            current_offset = current_offset + self.step
        return itertools.chain(*generators)


class QuestionHTMLFetcher(object):

    def __init__(self, session, username, step=10, **additional_parameters):
        self._session = session
        self._username = username
        self.step = step
        self._additional_parameters = additional_parameters

    @property
    def _uri(self):
        return 'profile/{0}/questions'.format(self._username)

    def _query_params(self, start_at):
        parameters = {'low': start_at, 'leanmode': 1}
        parameters.update(self._additional_paramters)
        return parameters

    def fetch(self, start_at):
        response = self._session.okc_get(self._uri,
                                         params=self._query_params(start_at))
        return response.content.decode('utf8')


class QuestionProcessor(object):

    _question_xpb = xpb.div.with_class('question')

    def __init__(self, session, id_to_existing=None):
        self._session = session
        self.id_to_existing = id_to_existing or {}

    def _process_question_element(self, question_element):
        id_ = question_element.attrib['data-qid']
        text = xpb.div.with_class('qtext').p.get_text_(question_element).strip()
        answer = None
        explanation = None
        if 'not_answered' not in question_element.attrib['class']:
            answer = xpb.span.attribute_contains('id', 'answer_target').\
                     get_text_(question_element).strip()

            explanation = xpb.div.span.with_class('note').get_text_(question_element).strip()
        return Question(id_, text, answer, explanation)

    def process(self, tree):
        for question_element in self._question_xpb.apply_(tree):
                yield self._process_question_element(question_element)


class Profile(object):

    attributes = set(('username', 'id', 'age', 'location', 'match_percentage',
                      'enemy_percentage', 'rating', 'contacted'))

    def __init__(self, session, username, *args, **kwargs):
        self._session = session
        self.username = username
        self._question_fetcher = QuestionFetcher.build(session, username)
        self.questions = util.Fetchable(self._question_fetcher)

        for key, value in kwargs.items():
            setattr(self, key, value)

        self._initialize_fillable_traits()

    @util.cached_property
    def _profile_response(self):
        return self._session.get(
            'https://www.okcupid.com/profile/{0}'.format(self.username)
        ).content.decode('utf8')

    @util.cached_property
    def _profile_tree(self):
        return html.fromstring(self._profile_response)

    def message_request_parameters(self, content, thread_id):
        return {
            'ajax': 1,
            'sendmsg': 1,
            'r1': self.username,
            'body': content,
            'threadid': thread_id,
            'authcode': self.authcode,
            'reply': 1 if thread_id else 0,
            'from_profile': 1
        }

    @util.cached_property
    def authcode(self):
        return helpers.get_authcode(self._profile_response)

    @util.cached_property
    def picture_uris(self):
        pics_request = self._session.okc_get('profile/{0}/photos?cf=profile'.format(self.username))
        pics_tree = html.fromstring(pics_request.content.decode('utf8'))
        return xpb.div(id='album_0').img.select_attribute_('src', pics_tree)

    rating_xpath = xpb.ul.li.with_class('current-rating').select_attribute_('style')

    @util.cached_property
    def rating(self):
        rating_style = self.rating_xpath.one_(self._profile_tree)
        width_percentage = int(''.join(c for c in rating_style if c.isdigit()))
        return (width_percentage // 100) * 5

    @util.cached_property
    def age(self):
        return int(xpb.span(id='ajax_age').get_text_(self._profile_tree).strip())

    @util.n_partialable
    def message(self, message, thread_id=None):
        return helpers.MessageSender(self._session).send_message(self.username, message,
                                                                 self.authcode, thread_id)

    def _initialize_fillable_traits(self):
        self.pics = []
        self.questions = []
        self.traits = []
        self.essays = {
            'self summary': '',
            'life': '',
            'good at': '',
            'first things': '',
            'favorites': '',
            'six things': '',
            'thinking': '',
            'friday night': '',
            'private thing': '',
            'message me if': '',
            }
        self.looking_for = {
            'gentation': '',
            'ages': '',
            'near': '',
            'single': '',
            'seeking': '',
            }
        self.details = {
            'last online': '',
            'orientation': '',
            'ethnicity': '',
            'height': '',
            'body type': '',
            'diet': '',
            'smokes': '',
            'drinks': '',
            'drugs': '',
            'religion': '',
            'sign': '',
            'education': '',
            'job': '',
            'income': '',
            'relationship type': '',
            'offspring': '',
            'pets': '',
            'speaks': '',
            }

    def update_traits(self):
        """
        Fill `self.traits` the personality traits of this profile.
        """
        get_traits = self._session.get('http://www.okcupid.com/profile/{0}/personality'.format(self.username))
        tree = html.fromstring(get_traits.content.decode('utf8'))
        self.traits = tree.xpath("//div[@class = 'pt_row']//label/text()")

    def __repr__(self):
        return 'Profile("{0}")'.format(self.username)




'''
Sample script that downloads the profile pictures of all 5'6"-5'8" Buddhists
living in New York, NY who either do not smoke or are trying to quit.
'''
from requests import get

import okcupyd


u = okcupyd.User()
profiles = u.search(location='new york, ny', religion='buddhist',
                    height_min=66, height_max=68, looking_for='everybody',
                    smokes=['no', 'trying to quit'], count=1)

for profile in profiles:
    u.visit(profile)
    print("Downloading {0}'s pictures...".format(profile.username))
    for count, url in enumerate(profile.picture_uris, start=1):
        extension = url.split('.')[-1].split('?')[0]
        response = get(url, stream=True)
        filename = '{0}{1}.{2}'.format(profile.username, count, extension)
        okcupyd.save_file(filename, response.raw)
        del response



from collections import namedtuple
import logging

from lxml import html

from . import helpers
from . import util
from .profile import Profile
from .xpath import XPathBuilder, xpb


log = logging.getLogger(__name__)


def ThreadFetcher(session, mailbox_number):
    return util.StepObjectFetcher(ThreadHTMLFetcher(session, mailbox_number),
                                  ThreadProcessor(session))


class ThreadHTMLFetcher(object):

    def __init__(self, session, mailbox_number, step=30):
        self._session = session
        self._mailbox_number = mailbox_number
        self.step = step

    def _query_params(self, start_at):
        return {
            'low': start_at,
            'folder': self._mailbox_number,
            'infiniscroll': 1
        }

    def fetch(self, start_at):
        return self._session.decoded_okc_get('messages',
                                             params=self._query_params(start_at)).strip()


class ThreadProcessor(object):

    xpath_builder = XPathBuilder(relative=False).li.with_classes('thread','message')

    def __init__(self, session, id_to_existing=None):
        self._session = session
        self.id_to_existing = id_to_existing or {}

    def process(self, text_response):
        for thread_element in self.xpath_builder.apply_(html.fromstring(text_response)):
            yield self._process_thread_element(thread_element)

    def _process_thread_element(self, thread_element):
        thread_id = thread_element.attrib['data-threadid']
        if self.id_to_existing is not None and thread_id in self.id_to_existing:
            return self.id_to_existing[thread_id]

        correspondent_id = thread_element.attrib['data-personid']

        correspondent = xpb.div.with_class('inner').a.with_class('photo').\
                        img.select_attribute_('alt', thread_element)[0].split()[-1]

        read = not 'unreadMessage' in thread_element.attrib['class']

        timestamp_span = xpb.span.with_class('timestamp').apply_(thread_element)
        date_updated_text = timestamp_span[0][0].text_content()
        date_updated = helpers.parse_date_updated(date_updated_text)

        return MessageThread(thread_id, correspondent, correspondent_id, read,
                             date_updated, session=self._session)


class MessageRetriever(object):

    def __init__(self, session, thread_id, read_messages=False):
        self._session = session
        self._read_messages = read_messages
        self.thread_id = thread_id

    @property
    def params(self):
        return {
            'readmsg': str(self._read_messages).lower(),
            'threadid': self.thread_id,
            'folder': 1
        }

    def thread_tree(self):
        messages_response = self._session.get('https://www.okcupid.com/messages',
                                              params=self.params)
        return html.fromstring(messages_response.content.decode('utf8'))

    user_xpath = xpb.div.with_class('profile_info').div.\
                 with_class('username').span.with_class('name').xpath

    def get_usernames(self, tree):
        try:
            them = tree.xpath(self.user_xpath)[0].text
        except IndexError:
            title_element = xpb.title.apply_(tree)[0]
            them = title_element.text.split()[-1]

        return helpers.get_my_username(tree), them

    def get_message_elements(self, thread_tree):
        return xpb.li.with_classes('to_me', 'from_me')._or.apply_(thread_tree)

    def get_messages(self):
        tree = self.thread_tree()
        me, them = self.get_usernames(tree)
        for message_element in self.get_message_elements(tree):
            if 'from_me' in message_element.attrib['class']:
                sender, recipient = me, them
            else:
                sender, recipient = them, me
            message_id = message_element.attrib['id'].split('_')[-1]
            message = xpb.div.with_class('message_body').apply_(message_element)
            if message_id == 'compose':
                continue
            content = None
            if message:
                message = message[0]
                content = message.text_content().replace(' \n \n', '\n').strip()

            yield Message(message_id, sender, recipient, content)


Message = namedtuple('Message', ('id', 'sender', 'recipient', 'content'))


class MessageThread(object):

    @classmethod
    def restore(cls, thread_id, correspondent, correspondent_id, read, date,
                session=None, messages=None):

        instance = cls(thread_id, correspondent, correspondent_id, read, date,
                       session)
        if messages is not None:
            instance.__dict__['messages'] = messages
        return instance

    def __init__(self, thread_id, correspondent, correspondent_id, read,
                 date, session=None):
        self._session = session
        self._message_retriever = MessageRetriever(session, thread_id)
        self.correspondent = correspondent
        self.correspondent_id = correspondent_id
        self.thread_id = thread_id
        self.read = read
        self.date = date
        self.reply = self.correspondent_profile.message(thread_id=self.thread_id)

    @property
    def redact_messages(self):
        return self.restore(self.thread_id, self.correspondent, self.read, self.date,
                            session=self._session,
                            messages=[self.redact_message(message)
                                      for message in self.messages])

    @staticmethod
    def redact_message(message):
        return Message(message.id, message.sender, message.recipient,
                       'x'*len(message.content))

    def __hash__(self):
        return hash(self.thread_id)

    def __eq__(self, other):
        return self.thread_id == other.thread_id

    @util.cached_property
    def messages(self):
        return self.get_messages()

    @util.cached_property
    def correspondent_profile(self):
        return Profile(self._session, self.correspondent)

    def get_messages(self):
        return list(self._message_retriever.get_messages())

    @property
    def refreshed_messages(self):
        if 'messages' in self.__dict__['messages']: del self.__dict__['messages']
        return self.messages

    @property
    def message_count(self):
        return len(self.messages)

    @property
    def initiator(self):
        if self.messages:
            return self.messages[0].sender

    @property
    def has_messages(self):
        return bool(self.messages)

    @property
    def got_response(self):
        return any(message.sender != self.initiator for message in self.messages)

    @property
    def as_dict(self):
        return {
            'thread_id': self.thread_id,
            'correspondent': self.correspondent,
            'correspondent_id': self.correspondent_id,
            'read': self.read,
            'date': self.date.strftime('%Y-%m-%d'),
            'messages': list(map(self.message_as_dict, self.messages))
        }

    @staticmethod
    def message_as_dict(message):
        return {
            'id': message.id,
            'sender': message.sender,
            'recipient': message.recipient,
            'content': message.content
        }

    def delete(self):
        return self._session.post('https://www.okcupid.com/mailbox',
                                  params=self.delete_params)

    @property
    def delete_params(self):
        return {
            'deletethread': 'DELETE',
            'mailaction': 3,
            'buddyname': self.correspondent,
            'r1': self.correspondent,
            'threadid': self.thread_id,
            'receiverid': self.correspondent_id,
            'folderid': 1,
            'body_to_forward': self.messages[-1].content
        }



'''
Sample script that visits a random profile from the quickmatch page,
and prints out most of the information in the profile.
'''

import okcupyd

u = okcupyd.User()
p = u.quickmatch() # p is an instance of the Profile class
u.visit(p)
p.update_traits() # only necessary for if you want to fill in p.traits

print('Profile of {0}'.format(p.username))
print('{0}: {1}'.format('Gender', p.gender))
print('{0}: {1}'.format('Age', p.age))
print('{0}: {1}'.format('Orientation', p.orientation))
print('{0}: {1}'.format('Location', p.location))
print('{0}: {1}%'.format('Match', p.match))
print('{0}: {1}%'.format('Enemy', p.enemy))
print('----------')
print('')

print('Traits')
print('----------')
for trait in p.traits:
    print(trait)
print('')

print('Essays')
print('----------')
for title, essay in p.essays.items():
    print('{0}: {1}'.format(title, essay))
print('')

print('Looking For')
print('----------')
for category, response in p.looking_for.items():
    print('{0}: {1}'.format(category, response))
print('')

print('Details')
print('----------')
for category, detail in p.details.items():
    print('{0}: {1}'.format(category, detail))



from . import util
from okcupyd import User


@util.use_cassette('user_no_picture')
def test_handle_no_pictures():
    assert isinstance(User().username, str)


@util.use_cassette('user_get_threads')
def test_get_inbox():
    user = User()
    assert len(user.inbox.items) == 1

    for message_thread in user.inbox:
        for message in message_thread.messages:
            assert hasattr(message, 'sender')


@util.use_cassette('access_profile_from_message_thread')
def test_message_thread_to_profile():
    profile = User().inbox[0].correspondent_profile
    assert profile.age
    assert profile.age > 18
    assert isinstance(profile.rating, int)


@util.use_cassette('user_count')
def test_user_search_count():
    search = User().search(count=1)
    import ipdb; ipdb.set_trace()
    assert len(User().search(count=1))



import datetime

import simplejson

from . import messaging
from .statistics import Statistics
from .user import User


# TODO: This code is pretty crappy and should be cleaned up.
def save_message_threads(filename, message_threads):
    with open(filename, 'w') as file_object:
        for thread in message_threads:
            file_object.write(simplejson.dumps(thread.as_dict) + '\n')


def load_message_threads(filename):
    with open(filename, 'r') as file_object:
        for line in file_object.readlines():
            thread_dictionary = simplejson.loads(line)
            thread_dictionary['date'] = datetime.datetime.strptime(thread_dictionary['date'], '%Y-%m-%d').date()
            thread_dictionary['messages'] = [messaging.Message(**message_dict) for message_dict in thread_dictionary['messages']]
            yield messaging.MessageThread.restore(**thread_dictionary)


def get_statistics_with_cached_data(user=None,
                                    attractiveness_file='attractiveness_cache.json',
                                    message_threads_file='message_threads.json'):
    user = user or User()
    mts = list(load_message_threads(message_threads_file))
    statistics = Statistics(user, mts)
    with open(attractiveness_file) as file_object:
        statistics._attractiveness_finder._cache = simplejson.loads(file_object.read())
    return statistics


def save_data_on_statistics_object(statistics,
                                   attractiveness_file='attractiveness_cache.json',
                                   message_threads_file='message_threads.json'):
    save_message_threads(message_threads_file, statistics.threads)
    with open(attractiveness_file, 'w') as file_object:
        file_object.write(simplejson.dumps(statistics._attractiveness_finder._cache))



import time
import logging

import requests

from . import helpers
from .settings import DELAY, USERNAME, PASSWORD


log = logging.getLogger(__name__)


class Session(requests.Session):

    @classmethod
    def login(cls, username=USERNAME, password=PASSWORD, headers=None):
        session = cls()
        helpers.login(session, {'username': username,
                                 'password': password}, headers)
        return session

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.timestamp = -DELAY

    def _throttle(self):
        while time.clock() - self.timestamp < DELAY:
            time.sleep(.5)
        self.timestamp = time.clock()

    def post(self, *args, **kwargs):
        self._throttle()
        response = super().post(*args, **kwargs)
        response.raise_for_status()
        return response

    def get(self, *args, **kwargs):
        self._throttle()
        response = super().get(*args, **kwargs)
        response.raise_for_status()
        return response



import copy
import os
import zlib

from six.moves import urllib
import simplejson
import vcr

from okcupyd import settings
from okcupyd import util

TESTING_USERNAME = 'username'
TESTING_PASSWORD = 'password'
WBITS = 16 + zlib.MAX_WBITS


SHOULD_SCRUB = True


@util.n_partialable
def remove_headers(request, headers_to_remove=()):
    if not SHOULD_SCRUB:
        return request
    headers = copy.copy(request.headers)
    headers_to_remove = [h.lower() for h in headers_to_remove]
    keys = [k for k in headers if k.lower() in headers_to_remove]
    if keys:
        for k in keys:
            headers.pop(k)
        request.headers = headers
    return request


def scrub_request_body(request):
    if not SHOULD_SCRUB:
        return request
    if not urllib.parse.urlsplit(request.uri).path == '/login':
        return request
    request.body = scrub_query_string(request.body)
    return request


def scrub_query_string(query_string):
    request_dict = urllib.parse.parse_qs(query_string)
    if 'password' not in request_dict:
        return query_string

    for key in request_dict:
        request_dict[key] = request_dict[key][0]
    request_dict['username'] = TESTING_USERNAME
    request_dict['password'] = TESTING_PASSWORD
    return urllib.parse.urlencode(request_dict)


def gzip_string(incoming):
    if isinstance(incoming, str) and bytes is not str:
        incoming = bytes(incoming, 'utf8')
    else:
        incoming = incoming.encode('utf8')
    compress_object = zlib.compressobj(6, zlib.DEFLATED, WBITS)
    start = compress_object.compress(incoming)
    end = compress_object.flush()
    return ''.join([start, end])


def scrub_login_response(response):
    if not SHOULD_SCRUB:
        return response
    response = response.copy()
    if 'Set-Cookie' in response['headers']:
        del response['headers']['Set-Cookie']
    try:
        decompressed = zlib.decompress(response['body']['string'], WBITS).decode('utf8')
    except:
        return response

    try:
        response_dict = simplejson.loads(decompressed)
    except:
        response['body']['string'] = gzip_string(decompressed.replace(settings.USERNAME, TESTING_USERNAME))
        return response

    if 'screenname' not in response_dict:
        return response
    if response_dict['screenname'] is not None:
        response_dict['screenname'] = TESTING_USERNAME
    response_dict['userid'] = 1
    response_dict['thumbnail'] = ''
    response['body']['string'] = gzip_string(simplejson.dumps(response_dict))
    return response


before_record = util.compose(scrub_request_body,
                             remove_headers(headers_to_remove=('Set-Cookie',
                                                               'Cookie')))


def match_search_query(left, right):
    left_filter = set([value for param_name, value in left.query
                               if 'filter' in param_name])
    right_filter = set([value for param_name, value in right.query
                                if 'filter' in param_name])
    left_rest = set([(param_name, value) for param_name, value in left.query
                     if 'filter' not in param_name])
    right_rest = set([(param_name, value) for param_name, value in right.query
                      if 'filter' not in param_name])
    return left_filter == right_filter and left_rest == right_rest


def body_as_query_string(left, right):
    try:
        return urllib.parse.parse_qs(left.body) == urllib.parse.parse_qs(right.body)
    except:
        return left.body == right.body


okcupyd_vcr = vcr.VCR(match_on=('path', 'method', 'match_search_query',
                                'body_as_query_string'),
                      before_record=before_record,
                      before_record_response=scrub_login_response,)
okcupyd_vcr.register_matcher('body_as_query_string', body_as_query_string)
okcupyd_vcr.register_matcher('match_search_query', match_search_query)



def cassette_path(cassette_name):
    return os.path.join(os.path.dirname(__file__),
                        'vcr_cassettes', '{0}.yaml'.format(cassette_name))

@util.n_partialable
def use_cassette(cassette_name, *args, **kwargs):
    return okcupyd_vcr.use_cassette(cassette_path(cassette_name), *args, **kwargs)



from okcupyd import util


@util.n_partialable
def crazy(a, self):
    return a + self.starter


class HasPartialableMethods(object):

    def __init__(self, starter):
        self.starter = starter
        self.on_instance = crazy

    @util.n_partialable
    def partialable(self, x, y=0):
        return self.starter * x * y

    on_class = crazy(1)


def test_partialable_method_behavior():
    assert HasPartialableMethods(2).partialable(y=2)(1) == 4
    hpm = HasPartialableMethods(2)
    assert hpm.on_instance(2, hpm) == 4
    assert hpm.on_class() == 3


def test_partialable_with_kwargs_taking_function():
    @util.n_partialable
    def kwarg_taking_function(arg, **kwargs):
        kwargs['k'] = arg
        return kwargs

    assert kwarg_taking_function(a=14)(2) == {'k': 2, 'a': 14}



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="okcupyd",
    version="0.2.0",
    packages=find_packages(),
    install_requires=['lxml', 'requests', 'simplejson'],
    tests_require=['tox', 'pytest', 'mock', 'contextlib2', 'six', 'vcrpy'],
    package_data={'': ['*.md', '*.rst']},
    author="Ivan Malison",
    author_email="ivanmalison@gmail.com",
    description="A package for interacting with OKCupid.com",
    license="MIT",
    keywords="python okcupid",
    url="https://github.com/IvanMalison/okcupyd",
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
    ],
)



import collections
import functools
import getpass
import inspect
import importlib
import itertools
import logging
import shutil

from . import settings


BASE_URI = 'https://www.okcupid.com/'


class n_partialable(object):

    @staticmethod
    def arity_evaluation_checker(function):
        is_class = inspect.isclass(function)
        if is_class:
            function = function.__init__
        function_info = inspect.getargspec(function)
        function_args = function_info.args
        if is_class:
            # This is to handle the fact that self will get passed in automatically.
            function_args = function_args[1:]
        def evaluation_checker(*args, **kwargs):
            kwarg_keys = set(kwargs.keys())
            if function_info.keywords == None:
                acceptable_kwargs = function_args[len(args):]
                # Make sure that we didn't get an argument we can't handle.
                assert kwarg_keys.issubset(acceptable_kwargs)
            needed_args = function_args[len(args):]
            if function_info.defaults:
                needed_args = needed_args[:-len(function_info.defaults)]
            return not needed_args or kwarg_keys.issuperset(needed_args)
        return evaluation_checker

    @staticmethod
    def count_evaluation_checker(count):
        def function(*args, **kwargs):
            return len(args) >= count
        return function

    def __init__(self, function, evaluation_checker=None, args=(), kwargs=None):
        self.function = function
        self.evaluation_checker = (evaluation_checker or
                                   self.arity_evaluation_checker(function))
        self.args = args
        self.kwargs = kwargs or {}
        self.__name__ = function.__name__

    def __call__(self, *args, **kwargs):
        new_args = self.args + args
        new_kwargs = self.kwargs.copy()
        new_kwargs.update(kwargs)
        if self.evaluation_checker(*new_args, **new_kwargs):
            return self.function(*new_args, **new_kwargs)
        else:
            return type(self)(self.function, self.evaluation_checker,
                              new_args, new_kwargs)

    def __get__(self, obj, obj_type):
        bound = type(self)(self.function, self.evaluation_checker,
                           args=self.args + (obj,), kwargs=self.kwargs)
        setattr(obj, self.function.__name__, bound)
        return bound


n_partialable = n_partialable(n_partialable)


def makelist(value):
    if not isinstance(value, list):
        if isinstance(value, collections.Iterable):
            return list(value)
        else:
            return [value]
    return value


def makelist_decorator(function):
    @functools.wraps(function)
    def wrapped(arg):
        return function(makelist(arg))

    return wrapped


class cached_property(object):

    def __init__(self, func):
        self.__doc__ = getattr(func, '__doc__')
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = self.func(obj)
        setattr(obj, self.func.__name__, value)
        return value


def _compose2(f, g):
    return lambda *args, **kwargs: f(g(*args, **kwargs))


@n_partialable(evaluation_checker=n_partialable.count_evaluation_checker(2))
def compose_with_joiner(joiner, *functions):
    return functools.reduce(joiner, functions)


compose_one_arg = compose_with_joiner(_compose2)


compose = compose_with_joiner(lambda f, g: _compose2(make_single_arity(f),
                                                force_args_return(g)))

def make_single_arity(function):
    @functools.wraps(function)
    def wrapped(args):
        return function(*args)
    return wrapped


def kwargs_make_single_arity(function):
    @functools.wraps(function)
    def wrapped(kwargs):
        return function(**kwargs)
    return wrapped


def args_kwargs_make_single_arity(function):
    @functools.wraps(function)
    def wrapped(val):
        (args, kwargs) = val
        return function(*args, **kwargs)
    return wrapped


def force_args_return(function):
    @functools.wraps(function)
    def wrapped(*args, **kwargs):
        value = function(*args, **kwargs)
        if not isinstance(value, collections.Iterable):
            value = (value,)
        return value
    return wrapped


def tee(*functions):
    def wrapped(*args, **kwargs):
        return tuple(function(*args, **kwargs) for function in functions)
    return wrapped


def enable_logger(log_name, level=logging.DEBUG):
    log = logging.getLogger(log_name)
    handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter('[%(name)s  %(asctime)s] %(message)s'))
    handler.setLevel(level)
    log.setLevel(level)
    log.addHandler(handler)


def get_credentials():
    if not settings.USERNAME:
        settings.USERNAME = input('username: ')
    if not settings.PASSWORD:
        settings.PASSWORD = getpass.getpass('password: ')


def add_command_line_options(add_argument, use_short_options=True):
    logger_args = ("--enable-logger",)
    credentials_args = ("--credentials",)
    if use_short_options:
        logger_args += ('-l',)
        credentials_args += ('-c',)
    add_argument(*logger_args, dest='enabled_loggers',
                 action="append", default=[], help="Enable the specified logger.")
    add_argument(*credentials_args, dest='credentials_modules',
                 action="append", default=[],
                 help="Use the specified credentials module to update "
                 "the values in okcupyd.settings.")


def handle_command_line_options(args):
    for enabled_log in args.enabled_loggers:
        enable_logger(enabled_log)
    for credentials_module in args.credentials_modules:
        update_settings_with_module(credentials_module)


def update_settings_with_module(module_name):
    module = importlib.import_module(module_name)
    if hasattr(module, 'USERNAME') and module.USERNAME:
        settings.USERNAME = module.USERNAME
    if hasattr(module, 'PASSWORD') and module.PASSWORD:
        settings.PASSWORD = module.PASSWORD
    if hasattr(module, 'AF_USERNAME') and module.AF_USERNAME:
        settings.AF_USERNAME = module.AF_USERNAME
    if hasattr(module, 'AF_PASSWORD') and module.AF_PASSWORD:
        settings.AF_PASSWORD = module.AF_PASSWORD


def save_file(filename, data):
    with open(filename, 'wb') as out_file:
        shutil.copyfileobj(data, out_file)


class Fetchable(object):

    def __init__(self, fetcher):
        self._fetcher = fetcher

    @cached_property
    def items(self):
        return list(self._fetcher.fetch())

    def refresh(self, use_existing=True):
        if 'items' in self.__dict__:
            self.items = list(self._fetcher.fetch(
                existing=self.items if use_existing else ()
            ))
        return self.threads

    def __iter__(self):
        return iter(self.items)

    def __getitem__(self, item):
        return self.items[item]


class StepObjectFetcher(object):

    def __init__(self, fetcher, processor, step=None):
        self._processor = processor
        self._fetcher = fetcher
        self.step = step or self._fetcher.step

    def fetch(self, start_at=0, get_at_least=None, id_to_existing=None):
        self._processor.id_to_existing = id_to_existing
        start_at_iterator = (range(start_at + 1, get_at_least + 1, self.step)
                             if get_at_least
                             else itertools.count(start_at + 1, self.step))
        generators = []
        for start_at in start_at_iterator:
            text_response = self._fetcher.fetch(start_at)
            if not text_response: break
            generators.append(self._processor.process(text_response))
        return itertools.chain(*generators)



import re

from lxml import html

from . import helpers
from . import util
from .messaging import ThreadFetcher
from .profile import Profile
from .search import SearchManager
from .session import Session


class User(object):

    @classmethod
    def with_credentials(cls, username, password):
        return cls(Session.login(username, password))

    def __init__(self, session=None):
        self._session = session or Session.login()
        profile_response = self._session.get('https://www.okcupid.com/profile')
        self._profile_tree = html.fromstring(profile_response.content.decode('utf8'))
        self.age, self.gender, self.orientation, self.status, self.location = helpers.get_additional_info(self._profile_tree)
        self.authcode = helpers.get_authcode(profile_response.content.decode('utf8'))

        self.username = helpers.get_my_username(self._profile_tree)
        self.questions = []
        self.visitors = []

        self._message_sender = helpers.MessageSender(self._session)
        self.inbox = util.Fetchable(ThreadFetcher(self._session, 1))
        self.outbox = util.Fetchable(ThreadFetcher(self._session, 2))
        self.drafts = util.Fetchable(ThreadFetcher(self._session, 4))

    def update_essay(self, essay_id, essay_body):
        form_data = {
            "essay_id": essay_id,
            "essay_body": essay_body,
            "authcode": self.authcode,
            "okc_api": 1
        }
        self._session.post('https://www.okcupid.com/profileedit2', data=form_data)

    def message(self, username, message_text):
        """
        Send a message to the username specified.
        Parameters
        ----------
        username : str or Profile
            Username of the profile that is being messaged.
        message_text : str
            Text body of the message.
        """
        thread_id = None
        if not isinstance(username, str):
            username = username.username
        for thread in sorted(set(self.inbox.items + self.outbox.items), key=lambda t: t.date,
                             reverse=True):
            if thread.correspondent.lower() == username.lower():
                thread_id = thread.thread_id
                break

        return self._message_sender.send_message(username, message_text, thread_id=thread_id)


    def search(self, **kwargs):
        return self.search_manager(**kwargs).get_profiles()

    def search_manager(self, **kwargs):
        kwargs.setdefault('gender', self.gender[0])
        kwargs.setdefault('looking_for', helpers.get_looking_for(self.gender, self.orientation))
        kwargs.setdefault('location', self.location)
        kwargs.setdefault('radius', 25)
        return SearchManager(**kwargs)

    def visit(self, username):
        """Visit another user's profile. Automatically update the
        `essays`, `details`, and `looking_for` attributes of the
        visited profile. Accept either a string or a Profile object as
        an argument. Note that unless your profile is set to browse
        anonymously on OKCupid, you are likely to show up on this
        user's visitors list.
        Parameters
        ---------
        username : str, Profile
            Username of the profile to visit. Can be either a string or a
            Profile object.
        update_pics : Bool
            Determines whether or not update_pics() is automatically
            called for this profile.
        Returns
        ---------
        Profile
            An instance of Profile containing the visited user's
            information.
        """
        if isinstance(username, Profile):
            prfl = username
        else: # string
            prfl = Profile(self._session, username)
        params = {
            'cf': 'leftbar_match',
            'leftbar_match': 1,
            }
        profile_request = self._session.post('http://www.okcupid.com/profile/{0}'.format(prfl.username), data=params)
        profile_tree = html.fromstring(profile_request.content.decode('utf8'))
        prfl.match, prfl.enemy = helpers.get_percentages(profile_tree)
        prfl.age, prfl.gender, prfl.orientation, prfl.status, prfl.location = helpers.get_additional_info(profile_tree)
        helpers.update_essays(profile_tree, prfl.essays)
        helpers.update_looking_for(profile_tree, prfl.looking_for)
        helpers.update_details(profile_tree, prfl.details)
        if prfl.id is None:
            prfl.id = helpers.get_profile_id(profile_tree)
        return prfl

    def update_visitors(self):
        """
        Update self.visitors with a Profile instance for each
        visitor on your visitors list.
        """
        get_visitors = self._session.get('http://www.okcupid.com/visitors')
        tree = html.fromstring(get_visitors.content.decode('utf8'))
        divs = tree.xpath("//div[@class = 'user_row_item clearfix  ']")
        for div in divs:
            name = div.xpath(".//a[@class = 'name']/text()")[0]
            age = int(div.xpath(".//div[@class = 'userinfo']/span[@class = 'age']/text()")[0])
            location = div.xpath(".//div[@class = 'userinfo']/span[@class = 'location']/text()")[0]
            match = int(div.xpath(".//p[@class = 'match_percentages']/span[@class = 'match']/text()")[0].replace('%', ''))
            enemy = int(div.xpath(".//p[@class = 'match_percentages']/span[@class = 'enemy']/text()")[0].replace('%', ''))
            self.visitors.append(Profile(self._session, name, age, location, match, enemy))

    def rate(self, profile, rating):
        """
        Rate a profile 1 through 5 stars. Profile argument may be
        either a Profile object or a string. However, if it is a
        string we must first visit the profile to get its id number.
        Parameters
        ----------
        profile : str or Profile
            The profile that you wish to rate.
        rating : str or int
            1 through 5 star rating that you wish to bestow.
        """
        if isinstance(profile, str):
            profile = self.visit(profile)
        parameters = {
            'target_userid': profile.id,
            'type': 'vote',
            'target_objectid': '0',
            'vote_type': 'personality',
            'score': rating,
        }
        self._session.post('http://www.okcupid.com/vote_handler',
                           data=parameters)

    def quickmatch(self):
        '''
        Return an instance of a Profile representing the profile on
        your Quickmatch page.
        Returns
        ----------
        Profile
        '''
        get_quickmatch = self._session.get('http://www.okcupid.com/quickmatch')
        tree = html.fromstring(get_quickmatch.content.decode('utf8'))
        # all of the profile information on the quickmatch page is hidden in
        # a <script> element, meaning that regex is unfortunately necessary
        for script in tree.iter('script'):
            if script.text is not None:
                search_result = re.search(r'[^{]"tuid" : "(\d+)', script.text)
                if search_result is not None:
                    id = search_result.group(1)
                # I'm sorry.
                broad_result = re.search(r'''"location"\s:\s"(.+?)".+
                                             "epercentage"\s:\s(\d{1,2}),\s
                                             "fpercentage"\s:\s(\d{1,2}),\s
                                             "tracking_age"\s:\s(\d{2}).+
                                             "sn"\s:\s"(.+?)",\s
                                             "percentage"\s:\s(\d{1,2})''',
                                             script.text, re.VERBOSE)
                if broad_result is not None:
                    location = broad_result.group(1)
                    enemy = int(broad_result.group(2))
                    age = int(broad_result.group(4))
                    username = broad_result.group(5)
                    match = int(broad_result.group(6))
        return Profile(self._session, username, age=age, location=location,
                       match=match, enemy=enemy, id=id)


    def __str__(self):
        return '<User {0}>'.format(self.username)
